// PLL MAX2870 PCB LTDZ RF Signal Source
// CW Beacon 21.01.24
// Freq REF 100 MHz
// Freq OUT 145 MHz
// http://infotex58.ru

#include <Adafruit_GFX.h>
#include <Adafruit_ST7735.h>
////////////////////////////////////////////////////
#define TFT_RST PA_5
#define TFT_DC PA_4
#define TFT_CS -1
#define TFT_MOSI PA_7
#define TFT_SCLK PA_6
#define PLL_CLK PB_15
#define PLL_DATA PB_14
#define PLL_LE PB_13
#define PLL_CE PB_12
#define SERIAL1_RX PA_10
#define SERIAL1_TX PA_9
#define KEY_UP PB_1
#define KEY_DW PA_3
#define KEY_L PA_1
#define KEY_R PB_0
#define KEY_OK PA_2
////////////////////////////////////////////////////
#define SPEED (18)      // WPM.
#define INTERVAL (0.3)  // Interval Min
#define DOTLEN (1200 / SPEED)
#define DASHLEN (3 * (1200 / SPEED))
#define PAUSE (INTERVAL * 60000)
////////////////////////////////////////////////////

byte data[]{
  0x01, 0x40, 0x00, 0x05,  //R5
  0x63, 0xDE, 0x80, 0xDC,  //R4
  0x01, 0x00, 0x01, 0x33,  //R3
  0xF9, 0x00, 0x40, 0x42,  //R2
  0x20, 0x03, 0x03, 0xE9,  //R1
  0x00, 0x2E, 0x03, 0x20   //R0
};

int upState, dwState, lState, rState, okState;

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_MOSI, TFT_SCLK, TFT_RST);
HardwareSerial Serial1(SERIAL1_RX, SERIAL1_TX);

void setup() {
  Serial1.begin(9600);
  pinMode(PLL_DATA, OUTPUT);
  pinMode(PLL_CLK, OUTPUT);
  pinMode(PLL_LE, OUTPUT);
  pinMode(PLL_CE, OUTPUT);

  pinMode(KEY_UP, INPUT_PULLUP);
  pinMode(KEY_DW, INPUT_PULLUP);
  pinMode(KEY_L, INPUT_PULLUP);
  pinMode(KEY_R, INPUT_PULLUP);
  pinMode(KEY_OK, INPUT_PULLUP);
  digitalWrite(PLL_CE, HIGH);

  tft.initR(INITR_MINI160x80);
  tft.setRotation(3);

  for (int i = 0; i < sizeof(data); i += 4) {
    digitalWrite(PLL_LE, LOW);
    shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (char(data[i])));
    shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (char(data[i + 1])));
    shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (char(data[i + 2])));
    shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (char(data[i + 3])));
    digitalWrite(PLL_LE, HIGH);
  }

  tft.setTextSize(2);
  tft.setCursor(20, 30);
  tft.fillScreen(ST7735_ORANGE);
  tft.setTextColor(ST7735_WHITE);
  tft.println("CW MAX2870");
  delay(3000);
}

void txOn() {
  digitalWrite(PLL_LE, LOW);
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0x63));
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0xDE));
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0x80));
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0xFC));
  digitalWrite(PLL_LE, HIGH);
}

void txOff() {
  digitalWrite(PLL_LE, LOW);
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0x63));
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0xDE));
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0x80));
  shiftOut(PLL_DATA, PLL_CLK, MSBFIRST, (0xDC));
  digitalWrite(PLL_LE, HIGH);
}

void loop() {
  
  tft.setCursor(0, 0);
  tft.fillScreen(ST7735_BLACK);
  tft.setTextColor(ST7735_GREEN);
  sendMsg((char *)"CQ CQ CQ RA4FHE RA4FHE BEACON LO23MF K K K");  // TXT Beacon Send
  delay(PAUSE);
}

void dash() {
  txOn();
  delay(DASHLEN);
  txOff();
  delay(DOTLEN);
}

void dot() {
  txOn();
  delay(DOTLEN);
  txOff();
  delay(DOTLEN);
}

void sendMsg(char *str) {
  int i;

  for (i = 0; i < strlen(str); i++) {
    tft.print(str[i]);
    Serial1.print(str[i]);
    switch (str[i]) {
      case 'A':
        dot();
        dash();
        break;
      case 'B':
        dash();
        dot();
        dot();
        dot();
        break;
      case 'C':
        dash();
        dot();
        dash();
        dot();
        break;
      case 'D':
        dash();
        dot();
        dot();
        break;
      case 'E':
        dot();
        break;
      case 'F':
        dot();
        dot();
        dash();
        dot();
        break;
      case 'G':
        dash();
        dash();
        dot();
        break;
      case 'H':
        dot();
        dot();
        dot();
        dot();
        break;
      case 'I':
        dot();
        dot();
        break;
      case 'J':
        dot();
        dash();
        dash();
        dash();
        break;
      case 'K':
        dash();
        dot();
        dash();
        break;
      case 'L':
        dot();
        dash();
        dot();
        dot();
        break;
      case 'M':
        dash();
        dash();
        break;
      case 'N':
        dash();
        dot();
        break;
      case 'O':
        dash();
        dash();
        dash();
        break;
      case 'P':
        dot();
        dash();
        dash();
        dot();
        break;
      case 'Q':
        dash();
        dash();
        dot();
        dash();
        break;
      case 'R':
        dot();
        dash();
        dot();
        break;
      case 'S':
        dot();
        dot();
        dot();
        break;
      case 'T':
        dash();
        break;
      case 'U':
        dot();
        dot();
        dash();
        break;
      case 'V':
        dot();
        dot();
        dot();
        dash();
        break;
      case 'W':
        dot();
        dash();
        dash();
        break;
      case 'X':
        dash();
        dot();
        dot();
        dash();
        break;
      case 'Y':
        dash();
        dot();
        dash();
        dash();
        break;
      case 'Z':
        dash();
        dash();
        dot();
        dot();
        break;
      case ' ':
        delay(DOTLEN * 5);
        break;
      case '.':
        dot();
        dash();
        dot();
        dash();
        dot();
        dash();
        break;
      case ',':
        dash();
        dash();
        dot();
        dot();
        dash();
        dash();
        break;
      case ':':
        dash();
        dash();
        dash();
        dot();
        dot();
        break;
      case '?':
        dot();
        dot();
        dash();
        dash();
        dot();
        dot();
        break;
      case '\'':
        dot();
        dash();
        dash();
        dash();
        dash();
        dot();
        break;
      case '-':
        dash();
        dot();
        dot();
        dot();
        dot();
        dash();
        break;
      case '/':
        dash();
        dot();
        dot();
        dash();
        dot();
        break;
      case '(':
      case ')':
        dash();
        dot();
        dash();
        dash();
        dot();
        dash();
        break;
      case '\"':
        dot();
        dash();
        dot();
        dot();
        dash();
        dot();
        break;
      case '@':
        dot();
        dash();
        dash();
        dot();
        dash();
        dot();
        break;
      case '=':
        dash();
        dot();
        dot();
        dot();
        dash();
        break;
      case '0':
        dash();
        dash();
        dash();
        dash();
        dash();
        break;
      case '1':
        dot();
        dash();
        dash();
        dash();
        dash();
        break;
      case '2':
        dot();
        dot();
        dash();
        dash();
        dash();
        break;
      case '3':
        dot();
        dot();
        dot();
        dash();
        dash();
        break;
      case '4':
        dot();
        dot();
        dot();
        dot();
        dash();
        break;
      case '5':
        dot();
        dot();
        dot();
        dot();
        dot();
        break;
      case '6':
        dash();
        dot();
        dot();
        dot();
        dot();
        break;
      case '7':
        dash();
        dash();
        dot();
        dot();
        dot();
        break;
      case '8':
        dash();
        dash();
        dash();
        dot();
        dot();
        break;
      case '9':
        dash();
        dash();
        dash();
        dash();
        dot();
        break;
    }
    delay(2 * DOTLEN);
  }
}